/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen.plugins.adapter;

import middlegen.RelationshipRole;
/**
 * A class to represent information about foreign keys, in a way that allows
 * easy access from within a velocity template (and other places).
 *
 * @author <a href="mailto:tfogwill@csir.co.za">Thomas Fogwill</a>
 * @created May 13, 2003
 */
public class AdapterFK {
   /**
    * Local table name - i.e. the name of the table that references the foreign
    * table/column
    */
   private String localTable;
   /**
    * Local column name - i.e. the name of the column that references the
    * foreign table/column
    */
   private String localColumn;
   /**
    * Foreign table name - i.e. the name of the table that is referenced by the
    * other (local) table/column
    */
   private String foreignTableSqlName;
   /**
    * Foreign table class name - i.e. the name of the table that is used to
    * generate a class name
    */
   private String foreignTableBaseClassName;
   /**
    * Foreign column name - i.e. the name of the column that is referenced by
    * the other (local) table/column
    */
   private String foreignColumn;

   /**
    * The relationshipRole object for this FK
    */
   private RelationshipRole relationshipRole;


   /**
    * Constructs a new AdapterFK object with the specified data.
    *
    * @param localTable Local table name - i.e. the name of the table that
    *      references the foreign table/column
    * @param localColumn Local column name - i.e. the name of the column that
    *      references the foreign table/column
    * @param foreignTableSqlName Foreign table name - i.e. the name of the table
    *      that is referenced by the other (local) table/column
    * @param foreignTableBaseClassName Foreign table name - i.e. the name of the
    *      table that is referenced by the other (local) table/column
    * @param foreignColumn Foreign column name - i.e. the name of the column
    *      that is referenced by the other (local) table/column
    * @param relationshipRole The relationshipRole object for this FK
    */
   AdapterFK(String localTable, String localColumn, String foreignTableSqlName,
         String foreignTableBaseClassName, String foreignColumn,
         RelationshipRole relationshipRole) {
      this.localTable = localTable;
      this.localColumn = localColumn;
      this.foreignTableSqlName = foreignTableSqlName;
      this.foreignTableBaseClassName = foreignTableBaseClassName;
      this.foreignColumn = foreignColumn;
      this.relationshipRole = relationshipRole;
   }


   /**
    * Sets the foreign column name - i.e. the name of the column that is
    * referenced by the other (local) table/column
    *
    * @param foreignColumnName The name of the foreign column
    */
   public void setForeignColumn(String foreignColumnName) {
      foreignColumn = foreignColumnName;
   }


   /**
    * Sets the foreign table name - i.e. the name of the table that is
    * referenced by the other (local) table/column
    *
    * @param foreignTableSqlName The sql name of the foreign table
    */
   public void setForeignTableSqlName(String foreignTableSqlName) {
      this.foreignTableSqlName = foreignTableSqlName;
   }


   /**
    * Sets the foreign table name - i.e. the name of the table that is
    * referenced by the other (local) table/column
    *
    * @param foreignTableBaseClassName The class name of the foreign table
    */
   public void setForeignTableBaseClassName(String foreignTableBaseClassName) {
      this.foreignTableBaseClassName = foreignTableBaseClassName;
   }


   /**
    * Sets the local column name - i.e. the name of the column that references
    * the foreign table/column
    *
    * @param localColumnName The name of the local column
    */
   public void setLocalColumn(String localColumnName) {
      localColumn = localColumnName;
   }


   /**
    * Sets the local table name - i.e. the name of the table that references the
    * foreign table/column
    *
    * @param localTableName The name of the local table
    */
   public void setLocalTable(String localTableName) {
      localTable = localTableName;
   }


   /**
    * Sets the relationshipRole for this FK
    *
    * @param relationshipRole The relationShipRole object for this FK
    */
   public void setRelationshipRole(RelationshipRole relationshipRole) {
      this.relationshipRole = relationshipRole;
   }


   /**
    * Gets the foreign column name - i.e. the name of the column that is
    * referenced by the other (local) table/column
    *
    * @return The name of the foreign column
    */
   public String getForeignColumn() {
      return foreignColumn;
   }


   /**
    * Gets the foreign table name - i.e. the name of the table that is
    * referenced by the other (local) table/column
    *
    * @return The name of the foreign table
    */
   public String getForeignTableSqlName() {
      return foreignTableSqlName;
   }


   /**
    * Gets the foreign table base class name - i.e. the name of the table that
    * is referenced by the other (local) table/column
    *
    * @return The name of the foreign table to be used as a base class name
    */
   public String getForeignTableBaseClassName() {
      return foreignTableBaseClassName;
   }


   /**
    * Gets the local column name - i.e. the name of the column that references
    * the foreign table/column
    *
    * @return The name of the local column
    */
   public String getLocalColumn() {
      return localColumn;
   }


   /**
    * Gets the local table name - i.e. the name of the table that references the
    * foreign table/column
    *
    * @return The name of the local table
    */
   public String getLocalTable() {
      return localTable;
   }


   /**
    * Gets the relationshipRole for this FK
    *
    * @return relationshipRole The relationShipRole object for this FK
    */
   public RelationshipRole getRelationshipRole() {
      return this.relationshipRole;
   }
}
