/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen.plugins.adapter;

import java.util.HashMap;
import java.net.URL;
import java.io.File;
import middlegen.FileProducer;
import middlegen.MiddlegenException;
import middlegen.javax.JavaPlugin;

/**
 * This plugin is used to generate the glue between the action and persistence
 * layers that are generated. To add support for a persistence layer to this
 * plugin you must provide a number of velocity templates that include the
 * method bodies the the generated delegate classes CRUD methods. The required
 * files are: NOTE: all methods are invoked with a the signature $voClass $vo,
 * these variables will have the correct object and type that are being used by
 * the action layer. Within the body method it is REQUIRED that the persistence
 * object be referenced as ${persistenceObjectName}. This allows the delegate to
 * perform any conversions that might be needed. DO NOT INCLUDE A RETURN
 * STATEMENT IN ANY METHOD IMPLEMENTATIONS {pluginName}-create.vm - this file
 * defines the create method body {pluginName}-edit.vm - this file defines the
 * edit method body {pluginName}-update.vm - this file defines the update method
 * body {pluginName}-remove.vm - this file defines the remove method body
 * {pluginName}-findAll.vm - this file defines the method body that will return
 * all entries in the table {pluginName}-select.vm - this file defines the
 * select method body, this method is passed in an empty instantiated list named
 * "retVals". This list is where the resulting objects should be stored, if a
 * conversion is needed the delegate will handle it. Optional files are:
 * {pluginName}-customClassMember.vm - you may define custom static class
 * methods and members in this file for use from your other methods, this is a
 * convinence only {pluginName}-imports.vm - you may define imports in this file
 * that will then be included in the generated delegate class
 * {pluginName}-persistenceLayerComment.vm - this provides you with a mechanism
 * to pass comments or strings through to the action layer that may be a
 * specific need to use your plugin (ex.
 *
 * @author Dylan Etkin
 * @created 02. may 2003
 * @web.ejb xdoclet comments for struts plugin) {pluginName}-copyFrom.vm - this
 *      file must be provided if the object that will be used by the action
 *      layer plugin may not be the object you use for persistence AND you want
 *      to put any plugin column or table specific checks to see if you should
 *      set a field on the object. This method copies from the persistence
 *      object to the value object NOTE: There is a default implementation of
 *      the copyFrom method if the file is not present {pluginName}-copyTo.vm -
 *      This file has the same conditions as the above. This method copies from
 *      the value object to the persistence object
 *      {pluginName}-velocity-variables.vm - this is a centralized place to
 *      define variables that you use in the your different templates NOTE: the
 *      class that is used as your container for data (persistence class) should
 *      be declared in the above file as a variable called "persistenceClass".
 *      This is used by the delegate to do conversions if needed. Adapter
 *      specific Ant task options: persistenceLayerName="cmp20" - this tells the
 *      adapter which persistence plugin it should generate for
 *      actionLayerName="struts" - this tells the apdater which action plugin is
 *      should generate for voPrefix="" - if the action plugin is going to
 *      manage the vo it is going to use to hold the data (ex. Form object in
 *      struts plugin) then this property lets the use specify the prefix to the
 *      table name voSuffix="Form" - same as above but a suffix to the object
 *      name NOTE: if voPrefix and voSuffix are not provided it is assumed that
 *      the persistenceClass is to be used by the action layer. In this case the
 *      action layer can remain object agnostic by getting the fully qualified
 *      object name by invoking getVoSyntax on the adapter plugin
 * @middlegen.plugin name="adapter"
 */
public class AdapterPlugin extends JavaPlugin {

   /**
    * This field stores the name of the persistence plugin the adapter generates
    * for
    */
   private String persistenceLayerName = null;

   /**
    * This field stores the name of the action plugin the adapter generates for
    */
   private String actionLayerName = null;

   /**
    * This field stores the prefix of the name of the value object used in the
    * adapter
    */
   private String voPrefix = null;

   /**
    * This field stores the prefix of the name of the value object used in the
    * adapter
    */
   private String voSuffix = null;

   /**
    * @todo-javadoc Describe the field
    */
   private HashMap comments = new HashMap();

   /**
    * This field is used to tell other plugins what the syntax of the value
    * object is that they should use
    */
   private HashMap voSyntax = new HashMap();


   /**
    * This plugin is used as an abstraction layer between action and persistence
    * layer plugins. This plugin generates a delegate class that can be used by
    * an action layer plugin to perform CRUD activities without having to know
    * any specific information about the persistence layer it is using. This
    * allows the plugin writer to write once and run against any other back-end
    * plugins that exist.
    */
   public AdapterPlugin() {
      super();
      setDisplayName("Adapter");
   }


   /**
    * This method sets the fully qualified object name that is to be used as the
    * value object in the delegate. This will be invoked during execution of
    * this plugin so that other plugins will be able to retrieve this syntax.
    *
    * @param syntax fully qualified object name to be used as a value object
    * @param tableName is the table the vo is related to
    */
   public void setVoSyntaxForTable(String syntax, String tableName) {
      voSyntax.put(tableName, syntax);
   }


   /**
    * Sets the prefix attribute of the value object used
    *
    * @param prefix The new prefix value
    */
   public void setVoPrefix(String prefix) {
      this.voPrefix = prefix;
   }


   /**
    * Sets the suffix attribute of the value object used
    *
    * @param suffix The new suffix value
    */
   public void setVoSuffix(String suffix) {
      this.voSuffix = suffix;
   }


   /**
    * Sets the PersistenceLayerName attribute of the AdapterPlugin object, this
    * defines which plugin the delegate will use for CRUD activities
    *
    * @param name The new PersistenceLayerName value
    */
   public void setPersistenceLayerName(String name) {
      this.persistenceLayerName = name;
   }


   /**
    * Sets the ActionLayerName attribute of the AdapterPlugin object which
    * defines which plugin will use the delegate that is generated.
    *
    * @param name The ActionLayerName value
    */
   public void setActionLayerName(String name) {
      this.actionLayerName = name;
   }


   /**
    * This method is used to get the fully qualified class name of the delegate
    * for the given table name
    *
    * @param tableName is the table you want the delegate class for
    * @return the fully qualified class name for the generated delegate
    */
   public String getDelegateClassNameForTable(String tableName) {
      return this.getPackage() + "." +
            actionLayerName.substring(0, 1).toUpperCase() +
            actionLayerName.substring(1) + tableName + "Delegate";
   }


   /**
    * This method returns a string that specifies the fully qualified object
    * name that the delegate is using as a value object. This should be used by
    * action layer plugins that do not generate their own value objects to
    * determine what vo they are working with.
    *
    * @param tableName is the table the vo is related to
    * @return this is the fully qualified object name that the delegate is using
    *      for its CRUD activities
    */
   public String getVoSyntaxForTable(String tableName) {
      String syntax = (String)voSyntax.get(tableName);
      if (syntax == null) {
         syntax = "";
      }
      return syntax;
   }


   /**
    * This method allows a layer of communication between the persistence layer
    * plugin and the action layer plugin. One use is to provide comments that
    * are used by other generaters (xdoclet for example) that are specific to
    * the persistence layer.
    *
    * @param tableName this is the key to get the correct comment for the given
    *      tableName. TableName is the result of JavaTable.getBaseClassName
    *      invocation.
    * @return this will return empty string if the file
    *      {persistenceLayerName}-persistenceLayerComment.vm was not provided as
    *      by the persistence layer. Else this will return the string defined in
    *      the above mentioned file with the syntax for the provided tableName.
    */
   public String getPersistenceLayerComment(String tableName) {
      StringBuffer comment = (StringBuffer)comments.get(tableName);
      if (comment == null) {
         return "";
      }
      return comment.toString();
   }


   /**
    * Gets the VoPrefix attribute of the AdapterPlugin object
    *
    * @return The VoPrefix value
    */
   public String getVoPrefix() {
      return this.voPrefix;
   }


   /**
    * Gets the VoSuffix attribute of the AdapterPlugin object
    *
    * @return The VoSuffix value
    */
   public String getVoSuffix() {
      return this.voSuffix;
   }


   /**
    * Gets the PersistenceLayerName attribute of the AdapterPlugin object
    *
    * @return The PersistenceLayerName value
    */
   public String getPersistenceLayerName() {
      return this.persistenceLayerName;
   }


   /**
    * Gets the ActionLayerName attribute of the AdapterPlugin object
    *
    * @return The ActionLayerName value
    */
   public String getActionLayerName() {
      return this.actionLayerName;
   }


   /**
    * This is used to see if the persistence layer has included an overriding
    * file for the copyTo implementation
    *
    * @return indicates if the file is present
    */
   public boolean isCopyToPresent() {
      return this.getRes(persistenceLayerName + File.separator
             + persistenceLayerName
             + "-copyTo.vm") != null;
   }


   /**
    * This is used to see if the persistence layer has included an overriding
    * file for the copyFrom implementation
    *
    * @return indicates if the file is present
    */
   public boolean isCopyFromPresent() {
      return this.getRes(persistenceLayerName + File.separator
             + persistenceLayerName
             + "-copyFrom.vm") != null;
   }


   /**
    * Gets the TableDecorator for this plugin
    *
    * @return The TableDecoratorClass for this plugin
    * @see middlegen.Plugin#getTableDecoratorClass()
    */
   public Class getTableDecoratorClass() {
      return AdapterTable.class;
   }


   /**
    * This is used by the plugin to set the comment provided in the file
    * {persistenceLayerName}-persistenceLayerComment.vm so that the information
    * will be available for other plugins. Each invocation adds a newline to the
    * generated string.
    *
    * @param newComment - is the string set in the file
    *      {persistenceLayerName}-persistenceLayerComment.vm provided by the
    *      persistence layer implementer.
    * @param tableName this is the key to set the correct comment for the given
    *      tableName. TableName is the result of JavaTable.getBaseClassName
    *      invocation.
    */
   public void appendLineToCommentForTable(String newComment, String tableName) {
      StringBuffer comment = null;
      if (!comments.containsKey(tableName)) {
         comment = new StringBuffer("");
         comments.put(tableName, comment);
      }
      else {
         comment = (StringBuffer)comments.get(tableName);
      }
      if (!comment.toString().equals("")) {
         comment.append("\n");
      }
      comment.append(newComment);
   }


   /**
    * This method is used to validate that the specified plugins exist and that
    * the files required are present.
    *
    * @exception MiddlegenException Describe the exception
    */
   public void validate() throws MiddlegenException {
      if (persistenceLayerName == null || actionLayerName == null) {
         throw new MiddlegenException("No layer names defined for Adapter pluggin");
      }
      else if (getMiddlegen().getPlugin(actionLayerName) == null) {
         throw new MiddlegenException("There is no " + actionLayerName
                + " plugin defined");
      }
      else if (getMiddlegen().getPlugin(persistenceLayerName) == null) {
         throw new MiddlegenException("There is no " + persistenceLayerName
                + " plugin defined");
      }
      else {
         // Now check to see if all the plugins manditory files are present
         String pre = persistenceLayerName + '/'
                + persistenceLayerName + "-";
         if (this.getRes(pre + "create.vm") == null) {
            throw new MiddlegenException("Missing file " + pre
                   + "create.vm");
         }
         if (this.getRes(pre + "edit.vm") == null) {
            throw new MiddlegenException("Missing file " + pre
                   + "edit.vm");
         }
         if (this.getRes(pre + "update.vm") == null) {
            throw new MiddlegenException("Missing file " + pre
                   + "update.vm");
         }
         if (this.getRes(pre + "remove.vm") == null) {
            throw new MiddlegenException("Missing file " + pre
                   + "remove.vm");
         }
         if (this.getRes(pre + "select.vm") == null) {
            throw new MiddlegenException("Missing file " + pre
                   + "select.vm");
         }
         if (this.getRes(pre + "findAll.vm") == null) {
            throw new MiddlegenException("Missing file " + pre
                   + "findAll.vm");
         }
      }
   }


   /**
    * This implements the abstract method and registers the delegate.vm to
    * generated for each table.
    */
   protected void registerFileProducers() {
      addConfiguredFileproducer(new FileProducer(getDestinationDir(),
            actionLayerName.substring(0, 1).toUpperCase() +
            actionLayerName.substring(1) + "{0}Delegate.java",
            this.getRes("delegate.vm")));
      addConfiguredFileproducer(new FileProducer(getDestinationDir(),
            getPackage().replace('.', '/') + "/TranslatorUtil.java",
            this.getRes("translator.vm")));
   }


   /**
    * Gets the Res attribute of the AdapterPlugin object
    *
    * @param name Describe what the parameter does
    * @return The Res value
    * @todo-javadoc Write javadocs for method parameter
    */
   private URL getRes(String name) {
      return getClass().getResource(name);
   }

}
